/*
 * Implementation of 2-3 trees using the Virgil language.
 */
class node {
    // used to guide search
    field left: node;
    field middle: node;
    field right: node;
    field smallest: keytype;

    // to get around doing subtype tests
    method isLeaf(): boolean {
        return false;
    }

    // there are no dynamic casts in Virgil
    method asLeaf(): leaf {
        return null;
    }

    // return the node to which the search should go next
    method path(k: keytype): node {
        if ( k.lessthan(middle.smallest) )
            return left;
        else if ( right == null or k.lessthan(right.smallest) )
            return middle;
        else
            return right;
    }

    // compare two nodes
    final method lessthan(n: node): boolean {
        return smallest.lessthan(n.smallest);
    }
}

class leaf extends node {
    // only used in leaf nodes
    field key: keytype;
    field value: valuetype;

    constructor(k: keytype, v: valuetype) {
        this.key = k;
        this.value = v;
        this.smallest = k;
    }

    method isLeaf(): boolean {
        return true;
    }

    method asLeaf(): leaf {
        return this;
    }
}

class tree {
    field root: node;
    field entries: int;

    constructor() {
        root = new node();
    }

    method find(k: keytype): valuetype {
        if ( entries == 0 ) return null;
        if ( entries == 1 ) {
            local pos = root.left.asLeaf();
            if ( k.equals(pos.key) ) return pos.value;
            else return null;
        }

        local pos = root;
        while ( !pos.isLeaf() ) pos = pos.path(k);

        local leaf = pos.asLeaf();
        if ( k.equals(leaf.key) ) return leaf.value;
        else return null;
    }

    method insert(k: keytype, v: valuetype) {
        if ( entries == 0 ) {
            root.left = new leaf(k, v);
            entries++;
        } else if ( entries == 1 ) {
            local pos = root.left.asLeaf();
            if ( k.equals(pos.key) ) pos.value = v;
            else {
                root.middle = new leaf(k,v);
                entries++;
            }
        } else {
            // we may have to split the root node
            local sibling = recursive_insert(root, k, v);
            if ( sibling != null ) {
                local nr = new node();
                nr.left = root;
                nr.smallest = root.smallest;
                nr.right = sibling;
                root = nr;
            }
        }
    }

    /*
     * The recursive_insert method inserts a key and value into the
     * tree below the specified node. If a split of the node is
     * required (because the node acquired a 4th child), it
     * is split into two nodes. The original node retains the
     * two smallest of the 4 descendants and the new node is
     * given the two larger descendants and returned. This therefore
     * implements splitting recursively, without requiring each
     * node to have a parent pointer.
     */
    private method recursive_insert(pos: node, k: keytype, v: valuetype): node {
        if ( pos.isLeaf() ) {
            local leaf = pos.asLeaf();
            if ( leaf.key.equals(k) ) {
                // overwrite the old value.
                leaf.value = v;
                return null;
            }
            // introduce a new value to be inserted in the tree
            return new leaf(k, v);
        }
        else {
            return accommodate(pos, recursive_insert(pos.path(k), k, v));
        }
    }

    private method accommodate(pos: node, orphan: node): node {
        // no new nodes to accommodate
        if ( orphan == null ) return pos;
        // there is room for this new node
        if ( pos.right == null ) return order(pos, pos.left, pos.middle, orphan);
        // a new node must be created
        return balance(pos, pos.left, pos.middle, pos.right, orphan);
    }

    private method order(pos: node, l: node, m: node, o: node): node {
        // ASSUMPTION: pos.left == l on entry
        // ASSUMPTION: pos.middle == m on entry

        if ( o.lessthan(l) ) {
            // ordering o, l, m
            pos.left = o;
            pos.smallest = o.smallest;
            pos.middle = l;
            pos.right = m;
        } else if ( o.lessthan(m) ) {
            // ordering l, o, m
            pos.middle = o;
            pos.right = m;
        } else {
            // ordering l, m, o
            pos.right = o;
        }
        // no splitting, no new nodes
        return null;
    }

    private method balance(pos: node, l: node, m: node, r: node, o: node): node {
        local nn = new node();
        // ASSUMPTION: pos.left == l on entry
        // ASSUMPTION: pos.middle == m on entry

        if ( o.lessthan(m) ) {
            // ordering (o, l), m, r
            if ( o.lessthan(l) ) {
                // ordering o, l, m, r
                pos.left = o;
                pos.middle = l;
                nn.left = m;
                nn.right = r;
                pos.smallest = o.smallest;
            } else {
                // ordering l, o, m, r
                pos.middle = o;
                nn.left = m;
                nn.right = r;
            }
        } else if ( o.lessthan(r) ) {
            // ordering l, m, o, r
            nn.left = o;
            nn.right = r;
        } else {
            // ordering l, m, r, o
            nn.left = r;
            nn.right = o;
        }

        // fix up search data
        pos.right = null;
        nn.smallest = nn.left.smallest;

        return nn;
    }

}