/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Dec 30, 2005
 */
package vpc.core;

import vpc.types.Type;
import vpc.tir.TIRInterpreter;

/**
 * The <code>Value</code> class represents a program value, such as an integer, boolean,
 * record reference, etc, during the execution of a program. Instances of this class represent
 * the values computed during the initialization phase of building an application where the
 * constructors from each of the components in the application are executed by the
 * <code>TIRInterpreter</code> in order to initialize component fields and build the program's
 * heap.
 * <p/>
 * </p>
 * <code>Value</code> objects are immutable, but not cached. Reference equality is <b>not</b>
 * enough for a comparison of values.
 *
 * @author Ben L. Titzer
 * @see TIRInterpreter
 */
public abstract class Value {

    /**
     * The <code>dynType</code> field represents the dynamic type of this value. Since values
     * are immutable, the type is also immutable.
     */
    public final Type dynType;

    /**
     * The basic constructor for the <code>Value</code> type constructs a value with the
     * specified dynamic type.
     * @param t the dynamic type of this value
     */
    protected Value(Type t) {
        dynType = t;
    }
}
