/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Apr 9, 2006
 */
package vpc.core.concept;

import cck.parser.AbstractToken;
import vpc.core.CompoundDecl;
import vpc.core.Member;
import vpc.types.TypeName;

/**
 * The <code>Field</code> class represents a field declaration inside a class or component.
 * A field has a name as a string and a type.
 *
 * @author Ben L. Titzer
 */
public class Field extends Member {
    protected FieldRep fieldRep;
    protected String uniqueName;
    public int fieldIndex;

    public Field(CompoundDecl decl, AbstractToken tok, TypeName rt) {
        super(decl, tok, rt);
        uniqueName = tok.image;
    }

    public Field(CompoundDecl decl, AbstractToken tok, String un, TypeName rt) {
        super(decl, tok, rt);
        uniqueName = un;
    }

    public FieldRep getFieldRep() {
        return fieldRep;
    }

    public void setFieldRep(FieldRep rep) {
        fieldRep = rep;
    }

    public String getUniqueName() {
        return uniqueName;
    }

    public String getFullName() {
        return container.getTypeName() + ":" + name;
    }

    public String toString() {
        StringBuffer b = new StringBuffer("field ");
        b.append(name);
        b.append(": ");
        b.append(typename);
        return b.toString();
    }

    public interface FieldRep {
    }
}
