/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Sep 28, 2006
 */
package vpc.core.concept;

import cck.util.Arithmetic;
import vpc.core.Value;
import vpc.tir.expr.Operator;

/**
 * The <code>PrimConversion</code> definition.
 *
 * @author Ben L. Titzer
 */
public class PrimConversion {
    public static class Int32_Raw extends Operator.Op1 {
        final PrimRaw.IType result;
        final long mask;

        public Int32_Raw(PrimRaw.IType rt) {
            super(PrimInt32.TYPE, rt);
            mask = Arithmetic.getLongBitRangeMask(0, rt.width - 1);
            result = rt;
        }

        public Value apply1(Value v1) {
            return PrimRaw.toValue(result.width, PrimInt32.fromValue(v1) & mask);
        }
    }

    public static class Char_Int32 extends Operator.Op1 {
        public Char_Int32() {
            super(PrimChar.TYPE, PrimInt32.TYPE);
        }

        public Value apply1(Value v1) {
            return PrimInt32.toValue(PrimChar.fromValue(v1));
        }
    }

    public static class Int32_Char extends Operator.Op1 {
        public Int32_Char() {
            super(PrimInt32.TYPE, PrimChar.TYPE);
        }

        public Value apply1(Value v1) {
            return PrimChar.toValue((char)PrimInt32.fromValue(v1));
        }
    }

    public static class Char_Raw extends Operator.Op1 {
        final PrimRaw.IType result;
        final long mask;

        public Char_Raw(PrimRaw.IType rt) {
            super(PrimChar.TYPE, rt);
            mask = Arithmetic.getLongBitRangeMask(0, rt.width - 1);
            result = rt;
        }

        public Value apply1(Value v1) {
            return PrimRaw.toValue(result.width, PrimChar.fromValue(v1) & mask);
        }
    }

    public static class AdjustRaw extends Operator.Op1 {
        final PrimRaw.IType result;
        final long mask;

        public AdjustRaw(PrimRaw.IType t1, PrimRaw.IType t2) {
            super(t1, t2);
            result = t2;
            mask = Arithmetic.getLongBitRangeMask(0, t2.width - 1);
        }

        public Value apply1(Value v1) {
            return PrimRaw.toValue(result.width, mask & PrimRaw.fromValue(v1));
        }
    }

    public static class Raw_Char extends Operator.Op1 {

        public Raw_Char(PrimRaw.IType t1) {
            super(t1, PrimChar.TYPE);
        }

        public Value apply1(Value v1) {
            return PrimChar.toValue((char)PrimRaw.fromValue(v1));
        }
    }

    public static class Raw_Int32 extends Operator.Op1 {

        public Raw_Int32(PrimRaw.IType t1) {
            super(t1, PrimInt32.TYPE);
        }

        public Value apply1(Value v1) {
            return PrimInt32.toValue((int)PrimRaw.fromValue(v1));
        }
    }
}
