/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Mar 27, 2006
 */
package vpc.core.concept;

import vpc.core.Value;
import vpc.types.Capability;
import vpc.types.Type;

/**
 * The <code>PrimVoid</code> concept represents the void type which is used to
 * give a type (and a value) to loop statements, switch statements, calls to
 * functions without a return type, etc.
 *
 * @author Ben L. Titzer
 */
public class PrimVoid {

    public static PrimVoid.IType TYPE = new IType();
    public static Val VALUE = new Val();

    /**
     * The <code>PrimVoid.Type</code> class represents the void type that is valid
     * only as the return type of void methods.
     */
    public static final class IType extends Type {
        IType() {
            super("void", VOID);
        }

        public boolean canBeComparedTo(Type t) {
            return false;
        }
    }

    /**
     * The <code>Val</code> value represents the value returned by expressions that do not
     * have a value, such as a loop, a switch statement, or calling a void procedure. This
     * value is not meant to be assigned to fields, local variables, etc, but should only
     * arise in the intermediate evaluation of expressions.
     */
    public static class Val extends Value {
        Val() {
            super(TYPE);
        }

        public String toString() {
            return "void";
        }
    }
}
