/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Creation date: Jan 30, 2007
 */

package vpc.core.csr;

import vpc.tir.expr.Operator;
import vpc.core.concept.PrimInt32;
import vpc.core.Value;
import cck.util.Util;

/**
 * The <code>CSRArray</code> class implements the concept of the C source representation
 * of an array type. This includes the array type and the operators associated with
 * C-level arrays.
 *
 * @author Ben L. Titzer
 */
public class CSRArray {

    /**
     * The <code>Array</code> class represents a C type corresponding to an array.
     * Thought C pointers and C arrays are nearly interchangeable, the syntax of
     * declaring an array differs slightly.
     */
    public static class IType extends CSRType {
        public final CSRType elemType;

        public IType(CSRType et) {
            super(et + "[]");
            elemType = et;
        }

        public String getVarDecl(String varname) {
            return elemType + " " + varname + "[]";
        }

        public String getLocalDecl(String varname) {
            return elemType + " *" + varname;
        }
    }

    public static class GetElement extends Operator.Op2 implements Operator.Location {
        public final IType arrayType;

        public GetElement(IType t) {
            // TODO: do not use PrimInt32
            super(t, PrimInt32.TYPE, t.elemType);
            arrayType = t;
        }

        public Value apply2(Value v1, Value v2) throws Operator.Exception {
            throw Util.unimplemented();
        }

        public Operator getOperator() {
            return this;
        }

        public Operator setOperator() {
            return new SetElement(arrayType);
        }
    }

    public static class SetElement extends Operator.Op3 {
        public final IType arrayType;

        public SetElement(IType t) {
            // TODO: do not use PrimInt32
            super(t, PrimInt32.TYPE, t.elemType, t.elemType);
            arrayType = t;
        }

        public Value apply3(Value v1, Value v2, Value v3) throws Operator.Exception {
            throw Util.unimplemented();
        }
    }


}
