/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Creation date: Jan 30, 2007
 */

package vpc.core.csr;

import vpc.tir.expr.Operator;
import vpc.core.Value;
import cck.util.Util;

/**
 * @author Ben L. Titzer
 */
public class CSRPointer {

    /**
     * The <code>Pointer</code> class represents the C pointer type, which encapsulates
     * the idea of a pointer to a typed memory location.
     */
    public static class IType extends CSRType {
        public final CSRType ptype;

        public IType(CSRType pt) {
            super(pt + " *");
            ptype = pt;
        }
    }

    /**
     * The <code>VoidPtr</code> class represents a void (untyped) pointer to memory.
     * Such a pointer can be used to circumvent the (admittedly weak) C type system.
     */
    public static class VoidPtr extends CSRType {
        public VoidPtr() {
            super("void *");
        }
    }

    public static class GetPtr extends Operator.Op1 implements Operator.Location {
        public final IType ptrType;

        public GetPtr(CSRPointer.IType t) {
            super(t, t.ptype);
            ptrType = t;
        }

        public Value apply1(Value v1) throws Operator.Exception {
            throw Util.unimplemented();
        }

        public Operator getOperator() {
            return this;
        }

        public Operator setOperator() {
            return new SetPtr(ptrType);
        }
    }

    public static class SetPtr extends Operator.Op2 {
        public final IType ptrType;

        public SetPtr(CSRPointer.IType t) {
            super(t, t.ptype, t.ptype);
            ptrType = t;
        }

        public Value apply2(Value v1, Value v2) throws Operator.Exception {
            throw Util.unimplemented();
        }
    }

    public static class GetAddr extends Operator.Op1 {
        public final IType ptrType;

        public GetAddr(CSRPointer.IType t) {
            super(t.ptype, t);
            ptrType = t;
        }

        public Value apply1(Value v1) throws Operator.Exception {
            throw Util.unimplemented();
        }
    }
}
