/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Creation date: Jan 26, 2006
 */

package vpc.core.csr;

import vpc.types.Type;
import vpc.tir.expr.Operator;
import vpc.core.Value;

import java.util.*;
import cck.util.Util;

/**
 * @author Ben L. Titzer
 */
public class CSRType extends Type {


    protected String typename;

    protected CSRType(String tn) {
        super(tn);
        typename = tn;
    }

    public String toString() {
        return typename;
    }

    public boolean canBeComparedTo(Type t) {
        throw Util.unimplemented();
    }

    /**
     * The <code>Unit</code> class represents a C type that corresponds to a primitive,
     * or unit, type. Such types include <code>int</code>, <code>char</code>, etc.
     */
    public static class Unit extends CSRType {
        public Unit(String n) {
            super(n);
        }
    }

    /**
     * The <code>Void</code> class represents the C type <code>void</code>, which
     * is used only as the return types for procedures that do not return a value.
     */
    public static class Void extends CSRType {
        public Void() {
            super("void");
        }
    }

    public String getVarDecl(String varname) {
        return toString() + " " + varname;
    }

    public String getLocalDecl(String varname) {
        return getVarDecl(varname);
    }

    public static CSRArray.IType newArray(CSRProgram csr, CSRType inner) {
        return csr.getCachedType(new CSRArray.IType(inner));
    }

    public static CSRPointer.IType newPointer(CSRProgram csr, CSRType pt) {
        return csr.getCachedType(new CSRPointer.IType(pt));
    }

    public static CSRFunction.IType newFuncPtr(CSRProgram csr, List<CSRType> pt, CSRType rt) {
        return csr.getCachedType(new CSRFunction.IType(pt, rt));
    }

    public static class Coercion extends Operator.Op1 {
        public Coercion(Type f, CSRType r) {
            super(f, r);
        }
        public Value apply1(Value v) {
            throw Util.unimplemented();
        }
    }
}
