/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Dec 20, 2006
 */
package vpc.core.virgil;

import vpc.core.*;
import vpc.core.concept.Method;
import vpc.util.Hierarchy;
import java.util.HashMap;
import java.util.HashSet;

/**
 * The <code>Closure</code> class definition. The <code>Closure</code> class
 * represents the complete set of classes, methods, layouts, metaobjects,
 * heap, etc for the program. The closure is used to compute information
 * for global analyses and for the final code generation step.
 *
 * @author Ben L. Titzer
 */
public class Closure {

    public final Program program;

    public int maxClassID;

    public HashSet<Method> methods = new HashSet<Method>();

    public Hierarchy<VirgilClass> hierarchy = new Hierarchy<VirgilClass>();

    public HashMap<VirgilComponent, Heap.Layout> componentLayouts;

    public HashMap<VirgilClass, Heap.Layout> classLayouts;

    public HashMap<VirgilClass, Heap.Layout> metaLayouts;

    public Closure(Program p) {
        program = p;
        resetLayouts();
    }

    public Closure copy() {
        Closure nc = new Closure(program);
        nc.maxClassID = maxClassID;
        nc.methods = methods;
        nc.hierarchy = hierarchy;
        nc.componentLayouts = componentLayouts;
        nc.classLayouts = classLayouts;
        nc.metaLayouts = metaLayouts;
        return nc;
    }

    public Heap.Layout getLayout(CompoundDecl d) {
        if ( d instanceof VirgilComponent ) return getLayout((VirgilComponent)d);
        if ( d instanceof VirgilClass ) return getLayout((VirgilClass)d);
        return null;
    }

    public Heap.Layout getLayout(VirgilComponent c) {
        return componentLayouts.get(c);
    }

    public Heap.Layout getLayout(VirgilClass c) {
        return classLayouts.get(c);
    }

    public Heap.Layout getMetaLayout(VirgilClass c) {
        return metaLayouts.get(c);
    }

    public void resetLayouts() {
        componentLayouts = new HashMap<VirgilComponent, Heap.Layout>();
        classLayouts = new HashMap<VirgilClass, Heap.Layout>();
        metaLayouts = new HashMap<VirgilClass, Heap.Layout>();
    }
}
