/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created May 23, 2006
 */
package vpc.core.virgil;

import vpc.core.Heap;
import vpc.core.Value;
import vpc.core.concept.Method;
import cck.util.Util;

/**
 * The <code>VirgilDelegate</code> class represents the delegate concept in
 * the Virgil language.
 *
 * @author Ben L. Titzer
 */
public class VirgilDelegate {
    /**
     * The <code>Val</code> class represents a value that is a reference to a method. This reference
     * can be to a component method (which does not include a "this" reference), or to an instance
     * method of some object (which does include a "this"--a reference to the record representing
     * the object).
     */
    public static class Val extends Value {
        public final Heap.Record record;
        public final Method method;

        public Val(Heap.Record r, Method m) {
            super(m.getTypeName().getType());
            record = r;
            method = m;
        }

        public boolean equals(Object o) {
            if (!(o instanceof Val)) return false;
            Val meth = (Val) o;
            return meth.method == method && Heap.compareRecords(meth.record, record);
        }

        public String toString() {
            return "[" + Heap.recordName(record) + "," + method.getFullName() + "]";
        }
    }

    public static Val fromValue(Value v) {
        if ( v instanceof Val ) return (Val)v;
        throw Util.failure("Cannot convert " + v.getClass() + " value to delegate");
    }

    public static boolean isNull(Value v) {
        if (v == null) return true;
        if (v instanceof Val) return ((Val)v).method == null;
        return false;
    }

}
