/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Mar 24, 2006
 */
package vpc.core.virgil;

import vpc.core.concept.Reference;
import vpc.types.Type;
import vpc.types.TypeName;

/**
 * The <code>VirgilMetaClass</code> class represents the type of a meta-object
 * that implements the type information, including the virtual dispatch table,
 * for an object at runtime.
 *
 * @author Ben L. Titzer
 */
public class VirgilMetaClass {
    public static class IType extends Type {
        public final VirgilClass classDecl;
        protected final TypeName parent;

        public IType(VirgilClass cd, TypeName n, TypeName p) {
            super(n.toString(), OBJECT, REFERENCE, EQUALITY);
            classDecl = cd;
            parent = p;
        }

        public TypeName getParent() {
            return parent;
        }

        public IType getParentMetaClass() {
            if (parent == null) return null;
            return (IType) parent.getType();
        }

        public boolean isSubtypeOf(IType c) {
            for (IType w = this; w != null; w = w.getParentMetaClass()) {
                if (c == w) return true;
            }
            return false;
        }

        public boolean canBeComparedTo(Type t) {
            if (t == this) return true;
            if (t == Reference.NULL_TYPE) return true;
            if (!t.isObject()) return false;
            IType tc = (IType) t;
            return isSubtypeOf(tc) || tc.isSubtypeOf(this);
        }

    }
}
