/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Creation date: Feb 7, 2006
 */

package vpc.hil;

import vpc.core.BaseDecl;
import vpc.core.Value;
import vpc.core.concept.PrimChar;
import vpc.hil.parser.Token;
import vpc.tir.expr.Operator;
import vpc.util.HashList;

/**
 * The <code>Device</code> class represents a hardware device that has interrupts, memory
 * spaces, regions, and hardware registers.
 *
 * @author Ben L. Titzer
 */
public class DeviceDecl extends BaseDecl {
    public final HashList<String, Space> spaces = new HashList<String, Space>();
    public final HashList<String, Region> regions = new HashList<String, Region>();
    public final HashList<String, Register> registers = new HashList<String, Register>();
    public final HashList<String, Interrupt> interrupts = new HashList<String, Interrupt>();

    public DeviceDecl(Token n) {
        super(n);
    }

    public Space newSpace(Token n, Token w, Token t) {
        Space sp = new Space(n, w, t);
        spaces.add(sp.getName(), sp);
        return sp;
    }

    public Region newRegion(Token n, Token s, Token l, Token h) {
        Region r = new Region(n, s, l, h);
        regions.add(r.getName(), r);
        return r;
    }

    public Register newRegister(Token n, Token r, Token i) {
        Register reg = new Register(this, n, r, i);
        registers.add(reg.getName(), reg);
        return reg;
    }

    public Interrupt newInterrupt(Token n, Token in) {
        Interrupt i = new Interrupt(n, in);
        interrupts.add(i.getName(), i);
        return i;
    }

    public static class GetRegister extends Operator.Op0 implements Operator.Location {
        public final Register register;
        public GetRegister(Register r) {
            super(PrimChar.TYPE);
            register = r;
        }
        public Value apply0() throws RegisterAccessError {
            throw new RegisterAccessError(register);
        }

        public Operator getOperator() {
            return this;
        }

        public Operator setOperator() {
            return new SetRegister(register);
        }
    }

    public static class SetRegister extends Operator.Op1 {
        public final Register register;
        public SetRegister(Register r) {
            super(PrimChar.TYPE, PrimChar.TYPE);
            register = r;
        }
        public Value apply1(Value v1) throws RegisterAccessError {
            throw new RegisterAccessError(register);
        }
    }

    public static class RegisterAccessError extends Operator.Exception {
        public RegisterAccessError(Register r) {
            super("RegisterAccessError", "register access is not supported at compile time");
        }
    }
}
