/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Creation date: Feb 7, 2006
 */

package vpc.hil;

import cck.text.StringUtil;
import vpc.core.BaseDecl;
import vpc.core.concept.PrimRaw;
import vpc.hil.parser.Token;
import vpc.types.Type;
import vpc.types.Typeable;

/**
 * The <code>Register</code> class represents a hardware register declared for a device.
 *
 * @author Ben L. Titzer
 */
public class Register extends BaseDecl implements Typeable {

    public final DeviceDecl device;
    public final Token regionToken;
    public final Token index;
    protected Region region;

    public Register(DeviceDecl dev, Token n, Token r, Token i) {
        super(n);
        device = dev;
        regionToken = r;
        index = i;
    }

    public int getAbsoluteAddress() {
        if (region == null) region = device.regions.get(regionToken.image);
        int ind = StringUtil.evaluateIntegerLiteral(index.image);
        int base = region.getBaseAddress();
        return base + ind;
    }

    public Type getType() {
        // TODO: get the raw width from the address space (or region)
        return PrimRaw.getType(8);
    }
}
