/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package vpc.tir;

/**
 * The <code>TIRExprVisitor</code> class implements a visitor for expressions
 * in TIR code.
 *
 * @author Ben L. Titzer
 */
public abstract class TIRExprVisitor<R, E> {

    public abstract R visit(TIRExpr e, E env);

    public R visit(TIRNop b, E env) {
        return visit((TIRExpr) b, env);
    }

    public R visit(TIROperator b, E env) {
        return visit((TIRExpr) b, env);
    }

    public R visit(TIRCompare c, E env) {
        return visit((TIRExpr) c, env);
    }

    public R visit(TIRCompare.Equal c, E env) {
        return visit((TIRCompare) c, env);
    }

    public R visit(TIRCompare.NotEqual c, E env) {
        return visit((TIRCompare) c, env);
    }

    public R visit(TIRConst c, E env) {
        return visit((TIRExpr) c, env);
    }

    public R visit(TIRConst.Value c, E env) {
        return visit((TIRConst) c, env);
    }

    public R visit(TIRConst.String c, E env) {
        return visit((TIRConst) c, env);
    }

    public R visit(TIRLocal r, E env) {
        return visit((TIRExpr) r, env);
    }

    public R visit(TIRLocal.Get r, E env) {
        return visit((TIRLocal) r, env);
    }

    public R visit(TIRCall c, E env) {
        return visit((TIRExpr) c, env);
    }

    public R visit(TIRThrow c, E env) {
        return visit((TIRExpr) c, env);
    }

    public R visit(TIRLocal.Set l, E env) {
        return visit((TIRLocal) l, env);
    }

    public R visit(TIRReturn r, E env) {
        return visit((TIRExpr) r, env);
    }

    public R visit(TIRIfExpr e, E env) {
        return visit((TIRExpr) e, env);
    }

    public R visit(TIRSwitch e, E env) {
        return visit((TIRExpr) e, env);
    }

    public R visit(TIRBlock e, E env) {
        return visit((TIRExpr) e, env);
    }

    public R visit(TIRLoop e, E env) {
        return visit((TIRExpr) e, env);
    }

    public R visit(TIRLoop.For e, E env) {
        return visit((TIRLoop) e, env);
    }

    public R visit(TIRLoop.While e, E env) {
        return visit((TIRLoop) e, env);
    }

    public R visit(TIRLoop.DoWhile e, E env) {
        return visit((TIRLoop) e, env);
    }

    public R visit(TIRBlock.Break e, E env) {
        return visit((TIRExpr) e, env);
    }

    public R visit(TIRBlock.Continue e, E env) {
        return visit((TIRExpr) e, env);
    }
}
