/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package vpc.tir;

import cck.text.Printer;

public class TIRPrinter extends TIRExprVisitor<Object, Object> {

    private final Printer p;

    public TIRPrinter(Printer p) {
        this.p = p;
    }

    public Object visit(TIRExpr e, Object env) {
        p.print(e.toString());
        return null;
    }

    public Object visit(TIRIfExpr s) {
        p.print("if ( " + s.condition.toString() + " ) ");
        print(s.true_target);
        if (s.false_target != null) {
            p.print("else ");
            print(s.false_target);
        }
        return null;
    }

    public Object visit(TIRLoop.For l) {
        p.startblock("for");
        p.print("init: ");
        print(l.initial);
        p.println("cond: " + l.condition);
        p.print("body: ");
        print(l.body);
        p.print("update: ");
        print(l.update);
        p.endblock();
        return null;
    }

    public Object visit(TIRLoop.DoWhile l) {
        p.print("do ");
        print(l.body);
        p.print("while ( " + l.condition + " )");
        return null;
    }

    public Object visit(TIRLoop.While l) {
        p.print("while ( " + l.condition + " ) ");
        print(l.body);
        return null;
    }

    public Object visit(TIRSwitch s) {
        p.startblock("switch (" + s.expr.toString() + ")");
        for (TIRSwitch.Case c : s.cases) {
            p.print("case (");
            p.print(TIRUtil.exprsToString(c.value));
            p.print(") ");
            print(c.body);
        }

        if (s.defcase != null) {
            p.print("default ");
            print(s.defcase.body);
        }
        return null;
    }

    public Object visit(TIRBlock b) {
        p.startblock("seq");
        for (TIRExpr e : b.list) {
            print(e);
            p.println(";");
        }
        p.endblock();
        return null;
    }

    private void print(TIRExpr obj) {
        obj.accept(this, null);
    }
}
