/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Apr 18, 2006
 */
package vpc.tir.expr;

import java.util.HashMap;

/**
 * The <code>Precedence</code> class stores the precedence number for nesting
 * infix binary operators. This precedence number is used to determine when
 * parentheses are required in order to enforce the correct order of operations
 * when generating a textual representation of an operator tree.
 *
 * @author Ben L. Titzer
 */
public class Precedence {

    protected static final HashMap<Class, Integer> map = new HashMap<Class, Integer>();

    public static int PREC_UNKNOWN = -1;
    public static int PREC_TERM = 25;
    public static int PREC_DOT = 19;
    public static int PREC_ARROW = 19;
    public static int PREC_INDEX = 19;
    public static int PREC_APPLY = 17;
    public static int PREC_DEREF = 16;
    public static int PREC_PREFIX_INC = 16;
    public static int PREC_COMP = 15;
    public static int PREC_NOT = 15;
    public static int PREC_MINUS = 14;
    public static int PREC_PLUS = 14;
    public static int PREC_CAST = 13;
    public static int PREC_MULTIPLY = 12;
    public static int PREC_DIVIDE = 12;
    public static int PREC_MODULUS = 12;
    public static int PREC_ADD = 11;
    public static int PREC_SUBTRACT = 11;
    public static int PREC_SHIFT_LEFT = 10;
    public static int PREC_SHIFT_RIGHT = 10;
    public static int PREC_LESS_THAN = 9;
    public static int PREC_LESS_EQUAL = 9;
    public static int PREC_GREATER = 9;
    public static int PREC_GREATER_EQUAL = 9;
    public static int PREC_EQUAL = 8;
    public static int PREC_NOT_EQUAL = 8;
    public static int PREC_BITWISE_AND = 7;
    public static int PREC_BITWISE_XOR = 6;
    public static int PREC_BITWISE_OR = 5;
    public static int PREC_LOGICAL_AND = 4;
    public static int PREC_LOGICAL_OR = 3;
    public static int PREC_IF_EXPR = 2;
    public static int PREC_ASSIGN = 1;

    public static void register(Class c, int i) {
        map.put(c, i);
    }

    public static int get(Class c) {
        Integer i = map.get(c);
        if (i == null) return PREC_UNKNOWN;
        return i;
    }

}
