/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Dec 26, 2006
 */
package vpc.tir.opt.rma;

import cck.text.Printer;
import cck.text.TermUtil;
import vpc.core.concept.Method;
import vpc.tir.opt.rma.RMAnalyzer.TypeInfo;
import vpc.types.Type;
import java.util.HashSet;
import java.util.Map;

/**
 * The <code>RMPrinter</code> definition.
 *
 * @author Ben L. Titzer
 */
public class RMPrinter {
    protected Printer printer;

    public void printResults(RMAnalyzer rma) {
        TermUtil.printSeparator(78, "RMA Results");
        printer.startblock("methods");
        for (Method m : rma.visitedMethods)
            printer.println(m.getFullName());
        printer.endblock();
        for (Map.Entry<Type, TypeInfo> e : rma.typeInfo.entrySet()) {
            printTypeInfo(e.getValue());
        }
    }

    private void printTypeInfo(TypeInfo info) {
        printer.startblock("type " + info.type);
        printList(printer, "Members: ", info.fields);
        printList(printer, "Subtypes: ", info.subtypes);
        printList(printer, "Instances: ", info.instances);
        printer.endblock();
    }

    public static void printList(Printer printer, String name, HashSet<?> list) {
        printer.beginList(name);
        if (list != null) for (Object r : list) {
            printer.print(r.toString());
        }
        printer.endListln();
    }
}
