/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Dec 26, 2006
 */
package vpc.tir.opt.rma;

import cck.text.Printer;
import cck.util.Option;
import vpc.core.Program;
import vpc.model.Stage;
import java.io.PrintStream;
import java.io.IOException;

/**
 * The <code>TIR_RMA</code> definition.
 *
 * @author Ben L. Titzer
 */
public class TIR_RMA extends Stage {

    protected final Option.Bool DEBUG = options.newOption("rma-debug", false,
            "This option enables debugging information for the RMA (reachable " +
            "members analysis) module. The output generated consists of units " +
            "of work performed by the analysis, as well as the reachable members " +
            "of each class and component in the program.");
    protected final Option.Bool ORPHANS = options.newOption("orphans", true,
            "This option controls whether the compiler will support efficient " +
            "implementation of orphan classes by removing the object header " +
            "and devirtualizing all calls to orphan methods.");

    public void visitProgram(Program p) throws IOException {
        // create an analyzer and initialize the debug printer
        RMAnalyzer rma = new RMAnalyzer();
        boolean debug = DEBUG.get();
        if ( debug ) rma.printer = new Printer(new PrintStream(System.out));
        // analyze the program
        rma.debug = debug;
        rma.analyzeProgram(p);

        // create a transformer and transform the program
        RMTransformer tf = new RMTransformer(rma);
        tf.orphans = ORPHANS.get();
        tf.transform();

        // output the results
        if ( debug ) {
            RMPrinter pr = new RMPrinter();
            pr.printer = rma.printer;
            pr.printResults(rma);
        }
    }

}
