/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Dec 20, 2006
 */
package vpc.tir.stages;

import vpc.core.Heap;
import vpc.core.Program;
import vpc.core.virgil.VirgilClass;
import vpc.core.virgil.VirgilTypeSystem;
import vpc.model.Stage;
import vpc.types.Type;
import java.util.HashSet;

/**
 * The <code>LiveHierarchy</code> definition.
 *
 * @author Ben L. Titzer
 */
public class LiveHierarchy extends Stage {

    public void visitProgram(Program p) {
        HashSet<VirgilClass> liveClasses = new HashSet<VirgilClass>();
        for (Heap.Record r : p.heap.getRecords()) {
            Type t = r.layout.type;
            if ( VirgilTypeSystem.isClass(t) ) {
                VirgilClass c = ((VirgilClass.IType)t).getDecl();
                liveClasses.add(c);
            }
        }
        // build a new class hierarchy with only the live classes (and LUBs)
        p.closure.hierarchy = p.virgil.hierarchy.rebuildWithLUB(liveClasses);

    }
}
