/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Creation date: Jan 25, 2007
 */

package vpc.util;

import java.lang.reflect.Array;

/**
 * @author Ben L. Titzer
 */
public class ArrayUtil {

    public static <T> T[] append(T[] ol, T val) {
        T[] nl = newArray(ol, ol.length + 1);
        System.arraycopy(ol, 0, nl, 0, ol.length);
        nl[ol.length] = val;
        return nl;
    }

    public static <T> T[] prepend(T val, T[] ol) {
        T[] nl = newArray(ol, ol.length + 1);
        System.arraycopy(ol, 0, nl, 1, ol.length);
        nl[0] = val;
        return nl;
    }

    public static <T> T[] trunc(T[] ol, int nlen) {
        T[] nl = newArray(ol, nlen);
        System.arraycopy(ol, 0, nl, 1, ol.length);
        return nl;
    }

    public static <T>T[] newArray(T[] ol, int nlen) {
        Class et = ol.getClass().getComponentType();
        return (T[]) Array.newInstance(et, nlen);
    }

    public static <S, T> T[] map(S[] ol, Delegate<T, S> mapfun, Class<T> tt) {
        T[] nl = (T[]) Array.newInstance(tt, ol.length);
        for ( int cntr = 0; cntr < ol.length; cntr++ ) {
            nl[cntr] = mapfun.invoke(ol[cntr]);
        }
        return nl;
    }
}
