/**
 * Copyright (c) 2006, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created Sep 12, 2006
 */
package vpc.vst.tree;

import vpc.core.Value;
import vpc.core.concept.PrimRaw;
import vpc.types.Type;
import vpc.util.Maybe;
import vpc.vst.parser.Token;
import vpc.vst.visitor.VSTAccumulator;
import vpc.vst.visitor.VSTVisitor;

/**
 * "0x9"
 * <p/>
 * The <code>VSTRawLiteral</code> class represents a raw literal
 * (i.e. a binary, octal, or hexadecimal constant) in the abstract
 * syntax tree of the program.
 */
public class VSTRawLiteral extends VSTLiteral {

    public final Maybe<PrimRaw.Val> value;

    public VSTRawLiteral(Token tok) {
        super(tok);
        value = PrimRaw.Converter.convert(tok);
    }

    public PrimRaw.Val getValue() {
        return value.getValue();
    }

    public Value computeConstValue() {
        return value.getValue();
    }

    public Type getType() {
        return value.getValue().dynType;
    }

    public void accept(VSTVisitor v) {
        v.visit(this);
    }

    public <V> V accept(VSTAccumulator<V> a) {
        return a.visit(this);
    }
}
