/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package vpc.vst.visitor;

import vpc.vst.tree.*;

/**
 * The <code>VSTExprAccumulator</code> interface represents a visitor for expressions
 * in the abstract syntax tree that computes a value for each of the visited expressions.
 * This is useful for transforming the syntax tree into another form or reducing it.
 */
public interface VSTAccumulator<V> {
    public V visit(VSTAppExpr a);

    public V visit(VSTArrayInitializer e);

    public V visit(VSTAssign e);

    public V visit(VSTBinOp e);

    public V visit(VSTIndexExpr e);

    public V visit(VSTTypeQueryExpr e);

    public V visit(VSTTypeCastExpr e);

    public V visit(VSTBooleanLiteral e);

    public V visit(VSTCharLiteral e);

    public V visit(VSTIntLiteral e);

    public V visit(VSTRawLiteral e);

    public V visit(VSTStringLiteral e);

    public V visit(VSTNullLiteral e);

    public V visit(VSTThisLiteral e);

    public V visit(VSTNewArrayExpr e);

    public V visit(VSTNewObjectExpr e);

    public V visit(VSTCompoundAssign e);

    public V visit(VSTTernaryExpr e);

    public V visit(VSTUnaryOp e);

    public V visit(VSTPreOp p);

    public V visit(VSTPostOp p);

    public V visit(VSTMemberExpr e);

    public V visit(VSTVarUse e);

    public V visit(VSTComparison c);

    public V visit(VSTBlock b);

    public V visit(VSTBreakStmt b);

    public V visit(VSTContinueStmt c);

    public V visit(VSTDoWhileStmt s);

    public V visit(VSTEmptyStmt e);

    public V visit(VSTExprStmt s);

    public V visit(VSTForStmt s);

    public V visit(VSTIfStmt s);

    public V visit(VSTReturnStmt r);

    public V visit(VSTSwitchCase s);

    public V visit(VSTSwitchStmt s);

    public V visit(VSTLocalVarDecl d);

    public V visit(VSTWhileStmt s);

    public V visit(VSTSuperClause s);
}
