/**
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the University of California, Los Angeles nor the
 * names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package vpc.vst.visitor;

import vpc.vst.tree.*;

/**
 * The <code>VSTVisitor</code> interface represents the visitor pattern and is
 * implemented by classes wishing to traverse a syntax tree.
 *
 * @author Ben L. Titzer
 */
public interface VSTVisitor {

    public void visit(VSTAppExpr a);

    public void visit(VSTArrayInitializer a);

    public void visit(VSTAssign a);

    public void visit(VSTBinOp b);

    public void visit(VSTBlock b);

    public void visit(VSTBooleanLiteral b);

    public void visit(VSTBreakStmt b);

    public void visit(VSTCharLiteral c);

    public void visit(VSTClassDecl d);

    public void visit(VSTComparison c);

    public void visit(VSTComponentDecl d);

    public void visit(VSTCompoundAssign c);

    public void visit(VSTConstructorDecl c);

    public void visit(VSTContinueStmt c);

    public void visit(VSTDoWhileStmt s);

    public void visit(VSTIndexExpr a);

    public void visit(VSTEmptyStmt e);

    public void visit(VSTExprStmt s);

    public void visit(VSTFieldDecl d);

    public void visit(VSTMemberExpr a);

    public void visit(VSTForStmt s);

    public void visit(VSTIfStmt s);

    public void visit(VSTTypeQueryExpr e);

    public void visit(VSTTypeCastExpr e);

    public void visit(VSTIntLiteral i);

    public void visit(VSTRawLiteral i);

    public void visit(VSTMethodDecl d);

    public void visit(VSTParamDecl p);

    public void visit(VSTPreOp p);

    public void visit(VSTPostOp p);

    public void visit(VSTModule m);

    public void visit(VSTNewArrayExpr e);

    public void visit(VSTNewObjectExpr e);

    public void visit(VSTNullLiteral l);

    public void visit(VSTReturnStmt r);

    public void visit(VSTStringLiteral l);

    public void visit(VSTSuperClause c);

    public void visit(VSTSwitchCase c);

    public void visit(VSTSwitchStmt s);

    public void visit(VSTTernaryExpr e);

    public void visit(VSTThisLiteral l);

    public void visit(VSTTypeRef r);

    public void visit(VSTUnaryOp u);

    public void visit(VSTLocalVarDecl d);

    public void visit(VSTVarUse u);

    public void visit(VSTWhileStmt s);

}
